package imp.ringing;

import android.util.Log;

/**
 * A simple protocol which encodes 0 and 1 into one and two rings, respectively.
 */
public class OneTwoProtocol extends Protocol {
	private boolean firstSignal;
	private boolean secondSignal;
	
	/**
	 * Initializes the protocol. 
	 */
	public OneTwoProtocol(Parametrization params) {
		super(params);
	}

	@Override
	public void send(boolean b, boolean last) throws InterruptedException {
		// Ring once or twice, depending on b.
		for (int i = 0; i < (b ? 2 : 1); i++) {
			ring();
				
			Log.i("USER", "Sender: Latch");
			this.mutexUp();
			Log.i("USER", "Sender: Latch down");
		}
		
		// If the bit was false, wait for the timeout.
		if (!b) {
			Log.i("USER", "Sender: Sleep...");
			Thread.sleep(params.getDeltaD() + params.getDeltaH());
			Log.i("USER", "Sender: ...wakeup");
		}
	}
	
	@Override
	public void callingCallback() {
	};
	
	@Override
	public void callingHangupCallback() {
		// Make sure that the calling intent is off.
		hangup();
		Log.i("USER", "Sender: Callback");

		this.mutexDown();
	}
	
	@Override
	public void receivingCallback(String number) {
		Log.i("USER", "Sender: Immediate hangup");		
		hangup();
		
		// Count the number of signals within the time limit.
		if (!firstSignal) {
			Log.i("USER", "Recvr : First signal");
			firstSignal = true;
		} else {
			Log.i("USER", "Recvr : Second signal");
			secondSignal = true;
		}
	};
	
	@Override
	public void receivingHangupCallback() {
		this.mutexDown();
	};
	
	@Override
	public boolean receive() throws InterruptedException {
		firstSignal = false;
		secondSignal = false;
		
		// Wait for the first signal.
		this.mutexUp();
		
		// Wait for the second signal until the time limit elapses.
		Log.i("USER", "Recvr : Waiting for the second signal...");
		this.mutexUpWithTimer(params.getDMax() + params.getHMax());
		
		Log.i("USER", "Recvr : Timeout");
		Log.i("USER", "Recvr : Received " + (secondSignal ? "1" : "0"));
		
		return secondSignal;
	}
}
