package imp.ringing;

import android.util.Log;

import java.io.UnsupportedEncodingException;

/**
 * A class used to process a message an sends it.
 */
public class Sender implements Runnable {
	private Protocol protocol;
	private String message;
	private String number;
	private Coder coder;
	
	/**
	 * Initializes this class with basic parameters.
	 * @param protocol Protocol with which the message will be sent.
	 * @param message The message.
	 * @param number The receiver's phone number.
	 * @param coder The coder with which to encode the message.
	 */
	public Sender(Protocol protocol, String message, String number, Coder coder) {
		this.protocol = protocol;
		this.message = message;
		this.number = number;
		this.coder = coder;
	}

	/**
	 * Sends the message.
	 */
	@Override
	public void run() {
		try {
			send(this.message);
		} catch (InterruptedException e) {
			Dialogs.messageBox("Sending cancelled",
				"The sending process was interrupted. Your message was not sent successfully.");
		}
	}
	
	public void send(String s) throws InterruptedException {
		final boolean[] b;
		
		try {
			b = coder.encode(s);
			
			String t = "";
			for (int i = 0; i < b.length; i++)
				t += b[i] ? "1" : "0";
			Log.i("USER", "Sender: Message " + t);
			
		} catch (UnsupportedEncodingException e) {
			Dialogs.messageBox("Encoding problem", e.getMessage());
			return;
		}
		
		Main.callBackProtocol = protocol;
		protocol.setDialedNumber(number);
		protocol.setProtocolState(ProtocolState.Sending);

        Log.i("USER", "Sender: Sending");
		Throbber.activate();
        
		for (int i = 0; i < b.length; i++) {
			Log.i("USER", "Sender: Sending a bit (" + (b[i] ? "1" : "0") + ")");
			protocol.send(b[i], /* last */ (i == b.length - 1));			
			Log.i("USER", "Sender: Sent a bit");
		}
		
		Throbber.deactivate();
	}
}
