package imp.ringing;

import java.io.UnsupportedEncodingException;

/**
 * A coder which uses UTF-8 to encode and decode the message.
 */
public class Utf8Coder extends Coder {
	@Override
	public boolean[] encode(String s) throws UnsupportedEncodingException {
		byte[] bytes = s.getBytes("UTF-8");
		boolean[] ret = new boolean[8 * bytes.length];
		
		for (int i = 0; i < bytes.length; i++) {
			for (int j = 0; j < 8; j++)
				ret[i * 8 + j] = ((bytes[i] & (1 << j)) != 0);
		}
		
		return ret;
	}
	
	@Override
	public String decode(Boolean[] s) {
		byte[] bytes = new byte[s.length / 8];
		
		for (int i = 0; i < s.length / 8; i++) {
			bytes[i] = 0;
			
			for (int j = 0; j < 8; j++)
				if (s[i * 8 + j])
					bytes[i] |= (1 << j);
		}

		try {
			return new String(bytes, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			// Try fewer bytes.
			if (s.length >= 8) {
				Boolean[] fewerBits = new Boolean[s.length - 8];
				for (int i = 0; i < s.length - 8; i++)
					fewerBits[i] = s[i];
				
				return decode(fewerBits);
			} else {
				return "";
			}
		}
	}
}
